@@ -110,6 +110,7 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
     m_view = new DolphinView(url, this);
     connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_urlNavigator, SLOT(setUrl(KUrl)));
     connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_messageWidget, SLOT(hide()));
+    connect(m_view, SIGNAL(directoryLoadingCompleted()),        m_messageWidget, SLOT(hide()));
     connect(m_view, SIGNAL(writeStateChanged(bool)),            this, SIGNAL(writeStateChanged(bool)));
     connect(m_view, SIGNAL(requestItemInfo(KFileItem)),         this, SLOT(showItemInfo(KFileItem)));
     connect(m_view, SIGNAL(itemActivated(KFileItem)),           this, SLOT(slotItemActivated(KFileItem)));
@@ -277,6 +278,9 @@ void DolphinViewContainer::showMessage(const QString& msg, MessageType type)
     const int unwrappedWidth = m_messageWidget->sizeHint().width();
     m_messageWidget->setWordWrap(unwrappedWidth > size().width());
 
+    if (m_messageWidget->isVisible()) {
+        m_messageWidget->hide();
+    }
     m_messageWidget->animatedShow();
 }
 
@@ -459,7 +463,16 @@ void DolphinViewContainer::slotDirectoryLoadingCanceled()
 void DolphinViewContainer::slotUrlIsFileError(const KUrl& url)
 {
     const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url);
-    slotItemActivated(item);
+
+    // Find out if the file can be opened in the view (for example, this is the
+    // case if the file is an archive). The mime type must be known for that.
+    item.determineMimeType();
+    const KUrl& folderUrl = DolphinView::openItemAsFolderUrl(item, true);
+    if (!folderUrl.isEmpty()) {
+        m_view->setUrl(folderUrl);
+    } else {
+        slotItemActivated(item);
+    }
 }
 
 void DolphinViewContainer::slotItemActivated(const KFileItem& item)
